# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/13 15:50
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: test_logging_util.py

import os
import shutil
import time
import pytest
import datetime
from unittest.mock import patch

from agentuniverse.base.util.logging.logging_config import LoggingConfig
from agentuniverse_extension.logger.sls_sink import SlsSender, SlsSink

from tests.test_agentuniverse_extension.mock.logger.mock_log_client import LogClient




@patch('agentuniverse.logger.sls_sink.LogClient', new=LogClient)
def test_sls_log():
    sls_sender = SlsSender(LoggingConfig.sls_project,
                           LoggingConfig.sls_log_store,
                           LoggingConfig.sls_endpoint,
                           LoggingConfig.access_key_id,
                           LoggingConfig.access_key_secret,
                           LoggingConfig.sls_log_queue_max_size,
                           LoggingConfig.sls_log_send_interval)

    sls_sink = SlsSink(sls_sender)

    class Message:
        pass

    message = Message()
    message.record = {"time": datetime.datetime.now()}
    sls_sink(message)
    result = sls_sender.batch_send()
    assert result


if __name__ == "__main__":
    pytest.main([__file__, "-s"])



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/26 14:31
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: thread_with_result.py

from threading import Thread


class ThreadWithReturnValue(Thread):
    """A thread can save the target func exec result."""
    def __init__(self, group=None, target=None, name=None,
                 args=(), kwargs=None):
        super().__init__(group, target, name, args, kwargs)

        if kwargs is None:
            kwargs = {}
        self.kwargs = kwargs
        self.args = args
        self.target = target
        self._return = None
        self.error = None

    def run(self):
        """Run the target func and save result in _return."""
        if self.target is not None:
            try:
                self._return = self.target(*self.args, **self.kwargs)
            except Exception as e:
                self.error = e

    def result(self):
        """Wait for target func finished, then return the result or raise an
        error."""
        self.join()
        if self.error is not None:
            raise self.error
        return self._return



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/13 10:56
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: peer_planner.py
"""Peer planner module."""
from agentuniverse.agent.action.tool.tool_manager import ToolManager
from agentuniverse.agent.agent_manager import AgentManager
from agentuniverse.agent.agent_model import AgentModel
from agentuniverse.agent.input_object import InputObject
from agentuniverse.agent.output_object import OutputObject
from agentuniverse.agent.plan.planner.planner import Planner
from agentuniverse.base.util.logging.logging_util import LOGGER

default_sub_agents = {
    'planning': 'PlanningAgent',
    'executing': 'ExecutingAgent',
    'expressing': 'ExpressingAgent',
    'reviewing': 'ReviewingAgent',
}

default_jump_step = 'expressing'

default_eval_threshold = 60

default_retry_count = 2


class PeerPlanner(Planner):
    """Peer planner class."""

    def invoke(self, agent_model: AgentModel, planner_input: dict, input_object: InputObject) -> dict:
        """Invoke the planner.

        Args:
            agent_model (AgentModel): Agent model object.
            planner_input (dict): Planner input object.
            input_object (InputObject): Agent input object.
        Returns:
            dict: The planner result.
        """
        planner_config = agent_model.plan.get('planner')
        sub_agents = self.generate_sub_agents(planner_config)
        return self.agents_run(sub_agents, planner_config, planner_input, input_object)

    @staticmethod
    def generate_sub_agents(planner_config: dict) -> dict:
        """Generate sub agents.

        Args:
            planner_config (dict): Planner config object.
        Returns:
            dict: Planner agents.
        """
        agents = dict()
        for config_key, default_agent in default_sub_agents.items():
            config_data = planner_config.get(config_key, None)
            agents[config_key] = AgentManager().get_instance_obj(config_data if config_data else default_agent)
        return agents

    @staticmethod
    def build_expert_framework(planner_config: dict, input_object: InputObject):
        """Build expert framework for the given planner config object.

        Args:
            planner_config (dict): Planner config object.
            input_object (InputObject): Agent input object.
        """
        expert_framework = planner_config.get('expert_framework')
        if expert_framework:
            context = expert_framework.get('context')
            selector = expert_framework.get('selector')
            if selector:
                selector_result = ToolManager().get_instance_obj(selector).run(**input_object.to_dict())
                input_object.add_data('expert_framework', selector_result)
            elif context:
                input_object.add_data('expert_framework', context)

    def agents_run(self, agents: dict, planner_config: dict, agent_input: dict, input_object: InputObject) -> dict:
        """Planner agents run.

        Args:
            agents (dict): Planner agents.
            planner_config (dict): Planner config object.
            agent_input (dict): Planner input object.
            input_object (InputObject): Agent input object.
        Returns:
            dict: The planner result.
        """
        result: dict = dict()

        loopResults = list()
        planning_result = dict()
        executing_result = dict()
        expressing_result = dict()
        reviewing_result = dict()

        retry_count = planner_config.get('retry_count', default_retry_count)
        jump_step = planner_config.get('jump_step', default_jump_step)
        eval_threshold = planner_config.get('eval_threshold', default_eval_threshold)

        self.build_expert_framework(planner_config, input_object)

        planningAgent = agents.get('planning')
        executingAgent = agents.get('executing')
        expressingAgent = agents.get('expressing')
        reviewingAgent = agents.get('reviewing')

        for _ in range(retry_count):
            LOGGER.info(f"Start execution sub agents, retry_count is {_}.")
            if not planning_result or jump_step == "planning":
                if not planningAgent:
                    LOGGER.warn("no planning agent, use default.")
                    planning_result = OutputObject({"framework": [agent_input.get('input')]})
                else:
                    planning_result = planningAgent.run(**input_object.to_dict())

                LOGGER.info(f"Planning agent execution result is {planning_result.to_json_str()}.")
                input_object.add_data('planning_result', planning_result)

            if not executing_result or jump_step in ["planning", "executing"]:
                if not executingAgent:
                    LOGGER.warn("no executing agent, use default.")
                    executing_result = OutputObject({})
                else:
                    executing_result = executingAgent.run(**input_object.to_dict())

                LOGGER.info(f"Executing agent execution result is {executing_result.to_json_str()}.")
                input_object.add_data('executing_result', executing_result)

            if not expressing_result or jump_step in ["planning", "executing", "expressing"]:
                if not expressingAgent:
                    LOGGER.warn("no expression agent, use default.")
                    expressing_result = OutputObject({})
                else:
                    expressing_result = expressingAgent.run(**input_object.to_dict())

                LOGGER.info(f"Expressing agent execution result is {expressing_result.to_json_str()}.")
                input_object.add_data('expressing_result', expressing_result)

            if not reviewing_result or jump_step in ["planning", "executing", "expressing", "reviewing"]:
                if not reviewingAgent:
                    LOGGER.warn("no expression agent, use default.")
                    loopResults.append({
                        "planning_result": planning_result,
                        "executing_result": executing_result,
                        "expressing_result": expressing_result,
                        "reviewing_result": reviewing_result
                    })
                    result['result'] = loopResults
                    return result
                else:
                    reviewing_result = reviewingAgent.run(**input_object.to_dict())

                    LOGGER.info(f"Reviewing agent execution result is {reviewing_result.to_json_str()}.")
                    input_object.add_data('evaluator_result', reviewing_result)
                    if reviewing_result.get_data('score') and reviewing_result.get_data('score') >= eval_threshold:
                        loopResults.append({
                            "planning_result": planning_result,
                            "executing_result": executing_result,
                            "expressing_result": expressing_result,
                            "reviewing_result": reviewing_result
                        })
                        result['result'] = loopResults
                        return result
                    else:
                        loopResults.append({
                            "planning_result": planning_result,
                            "executing_result": executing_result,
                            "expressing_result": expressing_result,
                            "reviewing_result": reviewing_result
                        })
        result['result'] = loopResults
        return result



import os

from agentuniverse.agent_serve.web import web_booster
from agentuniverse.base.agentuniverse import AgentUniverse


def test_service():
    os.environ['OPENAI_API_KEY'] = 'you openai api key'
    AgentUniverse().start(config_path='../agent/config.toml')
    web_booster.start_web_server(bind="127.0.0.1:8002")


if __name__ == "__main__":
    # os.environ['OPENAI_API_KEY'] = 'you openai api key'
    AgentUniverse().start(config_path='../agent/config.toml')
    web_booster.start_web_server(bind="0.0.0.0:8002")




# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/19 18:18
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: executing_agent.py
"""Executing Agent module."""
import copy
from concurrent.futures import ThreadPoolExecutor, wait, ALL_COMPLETED
from typing import Optional, Any
from agentuniverse.agent.agent import Agent
from agentuniverse.agent.input_object import InputObject
from agentuniverse.agent.plan.planner.planner_manager import PlannerManager


class ExecutingAgent(Agent):
    """Executing Agent class."""

    executor: Optional[Any] = ThreadPoolExecutor(max_workers=10, thread_name_prefix="executing_agent")

    def input_keys(self) -> list[str]:
        """Return the input keys of the Agent."""
        return ['planning_result']

    def output_keys(self) -> list[str]:
        """Return the output keys of the Agent."""
        return ['executing_result']

    def parse_input(self, input_object: InputObject, planner_input: dict) -> dict:
        """Planner parameter parsing.

        Args:
            input_object(InputObject): agent parameter object
            planner_input(dict): Planner input
        Returns:
            dict: Planner input
        """
        planner_input['input'] = input_object.get_data('input')
        planner_input['framework'] = input_object.get_data('planning_result').get_data('framework')
        return planner_input

    def parse_result(self, planner_result: dict) -> dict:
        """Planner result parser.

        Args:
            planner_result(dict): Planner result
        Returns:
            dict: Agent result object.
        """
        llm_result = []
        executing_result = []
        futures = planner_result.get('futures')
        for future in futures:
            task_result = future.result()
            llm_result.append(task_result)
            executing_result.append({
                'input': task_result['input'], 'output': task_result['output']
            })

        return {'executing_result': executing_result, 'llm_result': llm_result}

    def execute(self, input_object: InputObject) -> dict:
        """Execute agent instance.

        Args:
            input_object(InputObject): agent parameter object
        Returns:
            dict: Agent result object.
        """
        planning_result = input_object.get_data('planning_result')
        framework = planning_result.get_data('framework', [])
        futures = []
        for task in framework:
            input_object_copy: InputObject = copy.deepcopy(input_object)
            input_object_copy.add_data('input', task)
            planner_input = super().pre_parse_input(input_object_copy)
            futures.append(
                self.executor.submit(
                    PlannerManager().get_instance_obj(self.agent_model.plan.get('planner').get('name')).invoke,
                    self.agent_model, planner_input, input_object_copy))
        wait(futures, return_when=ALL_COMPLETED)
        return self.parse_result({'futures': futures})



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/18 10:51
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: prompt.py
introduction = """
你是一个文本编辑专家，你非常擅长从繁杂的信息中提取关键信息。
"""

target = """
你的目标是根据提供的信息生成一个新的答案，用来回答一个新的问题。
"""

instruction = """
需要从不同角度、有条理的回答用户的问题。
仅仅使用提供的知识，专业、详细的回答用户的问题。
需要遵守下面的规则：
- 回答必须是总-分结构，第一段总论点，用一句话简要总结回答问题。第二段分论点，进行专业、有结构性、通顺流畅的阐述论点。
- 仅仅可以使用提供的知识来回答问题，如果字数允许，可以使用知识中的数值和数据，作为分论点的论据支撑。如果提供的知识并不足够回答问题，则可以回复"缺乏具体信息"等解释的话术。
- 不描述重复的信息、不做对问题没有帮助的回答、不说模糊的推测、不可以出现XXX、ABC等不明确的词语。
- 整体文风要简明扼要，重点突出，没有重复信息，必要时可以通过空行提升阅读体验，生成的字数小于500字。

之前的对话:
{chat_history}

背景信息是:
{background}

开始!
需要回答的问题是: {input}
--------------------------------------------------------------
请根据规则要求做出回答。
"""



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/20 17:34
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: test_memory.py
import unittest

from langchain.chains import ConversationChain, LLMChain
from langchain_core.prompts import PromptTemplate

from agentuniverse.agent.memory.chat_memory import ChatMemory
from agentuniverse.agent.memory.enum import MemoryTypeEnum
from agentuniverse.agent.memory.message import Message
from agentuniverse.llm.openai_llm import OpenAILLM

template = """
You are a chatbot having a conversation with a human.

Previous conversation:
{chat_history}

Human: {human_input}
Chatbot:
"""


class MemoryTest(unittest.TestCase):
    """
    Test cases for Memory class
    """

    def setUp(self) -> None:
        init_params = dict()
        init_params['model_name'] = 'gpt-3.5-turbo'
        init_params['temperature'] = 0.7
        init_params['max_retries'] = 2
        init_params['streaming'] = False

        init_params['llm'] = OpenAILLM(**init_params)
        init_params['memory_key'] = 'chat_history'
        init_params['max_tokens'] = '1024'
        init_params['type'] = MemoryTypeEnum.LONG_TERM
        self.message0 = Message(type='system', content='This is the system message')
        self.message1 = Message(type='human', content='Hi, my name is Edwin')
        self.message2 = Message(type='ai', content='Hello Edwin, nice to meet you.')
        init_params['messages'] = [self.message0, self.message1, self.message2]
        self.chat_memory = ChatMemory(**init_params)

    def test_summarize_memory_1(self) -> None:
        langchain_memory = self.chat_memory.as_langchain()
        langchain_memory.memory_key = 'history'
        llm_chain = ConversationChain(llm=langchain_memory.llm,
                                      memory=langchain_memory)
        conversation = llm_chain.predict(input='Who am I？')
        print(conversation)

    def test_summarize_memory_2(self) -> None:
        langchain_memory = self.chat_memory.as_langchain()
        prompt = PromptTemplate(
            input_variables=["chat_history", "human_input"], template=template
        )
        llm_chain = LLMChain(llm=langchain_memory.llm, prompt=prompt,
                             memory=langchain_memory)
        conversation = llm_chain.predict(human_input='Who am I？')
        print(conversation)

    def test_summarize_memory_3(self) -> None:
        langchain_memory = self.chat_memory.as_langchain()
        prompt = PromptTemplate(
            input_variables=["chat_history", "human_input"], template=template
        )
        llm_chain = LLMChain(llm=langchain_memory.llm, verbose=True,
                             memory=langchain_memory, prompt=prompt)
        res = llm_chain(inputs={'human_input': 'What is my name?'})
        print(res)

    def test_truncate_memory_1(self) -> None:
        self.chat_memory.type = MemoryTypeEnum.SHORT_TERM
        langchain_memory = self.chat_memory.as_langchain()
        prompt = PromptTemplate(
            input_variables=["chat_history", "human_input"], template=template
        )
        llm_chain = LLMChain(llm=langchain_memory.llm, prompt=prompt,
                             memory=langchain_memory)
        conversation = llm_chain.predict(human_input='Who am I？')
        print(conversation)

    def test_truncate_memory_2(self) -> None:
        self.chat_memory.type = MemoryTypeEnum.SHORT_TERM
        langchain_memory = self.chat_memory.as_langchain()
        prompt = PromptTemplate(
            input_variables=["chat_history", "human_input"], template=template
        )
        llm_chain = LLMChain(llm=langchain_memory.llm,
                             memory=langchain_memory, prompt=prompt)
        res = llm_chain({'human_input': 'What is my name?'})
        print(res)


if __name__ == '__main__':
    unittest.main()



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/22 14:30
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: reader.py
from abc import abstractmethod
from typing import List, Any

from pydantic import BaseModel

from agentuniverse.agent.action.knowledge.store.document import Document


class Reader(BaseModel):
    """The basic class for the knowledge reader."""

    @abstractmethod
    def load_data(self, *args: Any, **kwargs: Any) -> List[Document]:
        """Load data from the input params."""



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/12 19:22
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: prompt_model.py
"""Agent Prompt Model module."""
from typing import Optional

from pydantic import BaseModel


class AgentPromptModel(BaseModel):
    """Agent Prompt Model class."""

    introduction: Optional[str] = None
    target: Optional[str] = None
    instruction: Optional[str] = None

    def __add__(self, other):
        """Merge two objects into one object."""
        merged_object = AgentPromptModel()
        for key in set(self.__dict__.keys()).union(other.__dict__.keys()):
            value = getattr(self, key, None)
            if value is None:
                value = getattr(other, key, None)
            setattr(merged_object, key, value)
        return merged_object



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/19 19:37
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: expressing_agent.py
"""Expressing Agent module."""
from agentuniverse.agent.agent import Agent
from agentuniverse.agent.input_object import InputObject


class ExpressingAgent(Agent):
    """Expressing Agent class."""

    def input_keys(self) -> list[str]:
        """Return the input keys of the Agent."""
        return ['input']

    def output_keys(self) -> list[str]:
        """Return the output keys of the Agent."""
        return ['output']

    def parse_input(self, input_object: InputObject, planner_input: dict) -> dict:
        """Planner parameter parsing.

        Args:
            input_object(InputObject): agent parameter object
            planner_input(dict): Planner input
        Returns:
            dict: Planner input
        """
        planner_input['input'] = input_object.get_data('input')
        planner_input['background'] = self.build_background(input_object)
        return planner_input

    def parse_result(self, planner_result: dict) -> dict:
        """Planner result parser.

        Args:
            planner_result(dict): Planner result
        Returns:
            dict: Agent result object.
        """
        return planner_result

    def build_background(self, input_object: InputObject) -> str:
        """Build the background knowledge.

        Args:
            input_object(InputObject): agent parameter object
        Returns:
            str: Background knowledge.
        """
        executing_result = input_object.get_data('executing_result').get_data('executing_result', [])
        knowledge_list = []
        for execution in executing_result:
            knowledge_list.append("question:" + execution.get('input'))
            knowledge_list.append("answer:" + execution.get('output'))

        return '\n\n'.join(knowledge_list)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/25 14:58
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: rag_planner.py
"""Rag planner module."""
import asyncio
from langchain.chains import LLMChain
from langchain_core.memory import BaseMemory
from agentuniverse.agent.agent_model import AgentModel
from agentuniverse.agent.input_object import InputObject
from agentuniverse.agent.plan.planner.planner import Planner
from agentuniverse.agent.plan.planner.rag_planner.prompt import introduction, target, instruction
from agentuniverse.base.util.prompt_util import process_llm_token
from agentuniverse.llm.llm import LLM
from agentuniverse.prompt.prompt_model import AgentPromptModel


class RagPlanner(Planner):
    """Rag planner class."""

    def invoke(self, agent_model: AgentModel, planner_input: dict,
               input_object: InputObject) -> dict:
        """Invoke the planner.

        Args:
            agent_model (AgentModel): Agent model object.
            planner_input (dict): Planner input object.
            input_object (InputObject): Agent input object.
        Returns:
            dict: The planner result.
        """
        memory: BaseMemory = self.handle_memory(agent_model, planner_input)

        self.handle_action(agent_model, planner_input, input_object)

        llm: LLM = self.handle_llm(agent_model)

        self.handle_prompt(agent_model, planner_input)

        llm_chain = LLMChain(llm=llm.as_langchain(),
                             prompt=self.prompt.as_langchain(),
                             output_key=self.output_key, memory=memory)
        return asyncio.run(llm_chain.acall(inputs=planner_input))

    def handle_prompt(self, agent_model: AgentModel, planner_input: dict):
        """Prompt module processing.

        Args:
            agent_model (AgentModel): Agent model object.
            planner_input (dict): Planner input object.
        Returns:
            PromptTemplate: The prompt template.
        """
        profile: dict = agent_model.profile

        user_prompt_model: AgentPromptModel = AgentPromptModel(introduction=profile.get('introduction'),
                                                               target=profile.get('target'),
                                                               instruction=profile.get('instruction'))

        system_prompt_model: AgentPromptModel = AgentPromptModel(introduction=introduction, target=target,
                                                                 instruction=instruction)
        self.prompt.build_prompt_template(user_prompt_model, system_prompt_model,
                                          self.prompt_assemble_order)
        process_llm_token(self.prompt.as_langchain(), profile, planner_input)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/4/2 16:20
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: default_openai_llm.py
from typing import Any

from agentuniverse.llm.llm_output import LLMOutput
from agentuniverse.llm.openai_llm import OpenAILLM


class DefaultOpenAILLM(OpenAILLM):
    """The AgentUniverse default openai llm module.

    LLM parameters, such as name/description/model_name/max_tokens,
    are injected into this class by the default_openai_llm.yaml configuration.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def call(self, messages: list, **kwargs: Any) -> LLMOutput:
        """ The call method of the LLM.

        Users can customize how the model interacts by overriding call method of the LLM class.

        Args:
            messages (list): The messages to send to the LLM.
            **kwargs: Arbitrary keyword arguments.
        """
        return super().call(messages, **kwargs)

    async def acall(self, messages: list, **kwargs: Any) -> LLMOutput:
        """ The async call method of the LLM.

        Users can customize how the model interacts by overriding acall method of the LLM class.

        Args:
            messages (list): The messages to send to the LLM.
            **kwargs: Arbitrary keyword arguments.
        """
        return await super().acall(messages, **kwargs)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/15 10:05
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: memory.py
from typing import Optional

from langchain_core.memory import BaseMemory

from agentuniverse.agent.memory.enum import MemoryTypeEnum
from agentuniverse.base.component.component_base import ComponentBase
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.config.application_configer.application_config_manager import ApplicationConfigManager
from agentuniverse.base.config.component_configer.configers.memory_configer import MemoryConfiger


class Memory(ComponentBase):
    """The basic class for memory model.

    Attributes:
        name (Optional[str]): The name of the memory class.
        description (Optional[str]): The description of the memory class.
        type (MemoryTypeEnum): The type of the memory class including `long-term` and `short-term`.
        memory_key (Optional[str]): The name of the memory key in the prompt.
        max_tokens (int): The maximum number of tokens allowed in the memory.
    """

    name: Optional[str] = ""
    description: Optional[str] = None
    type: MemoryTypeEnum = None
    memory_key: Optional[str] = 'chat_history'
    max_tokens: int = 2000

    def __init__(self, **kwargs):
        super().__init__(component_type=ComponentEnum.MEMORY, **kwargs)

    def as_langchain(self) -> BaseMemory:
        """Convert the finCo memory class to the langchain memory class."""
        pass

    def set_by_agent_model(self, **kwargs) -> None:
        """ Assign values of parameters to the Memory model in the agent configuration."""
        if 'memory_key' in kwargs and kwargs['memory_key']:
            self.memory_key = kwargs['memory_key']
        if 'max_tokens' in kwargs and kwargs['max_tokens']:
            self.max_tokens = kwargs['max_tokens']

    def get_instance_code(self) -> str:
        """Return the full name of the memory."""
        appname = ApplicationConfigManager().app_configer.base_info_appname
        return f'{appname}.{self.component_type.value.lower()}.{self.name}'

    def initialize_by_component_configer(self, component_configer: MemoryConfiger) -> 'Memory':
        """Initialize the memory by the ComponentConfiger object.
        Args:
            component_configer(MemoryConfiger): the ComponentConfiger object
        Returns:
            Memory: the Memory object
        """
        if component_configer.name:
            self.name = component_configer.name
        if component_configer.description:
            self.description = component_configer.description
        if component_configer.type:
            self.type = next((member for member in MemoryTypeEnum if member.value == component_configer.type))
        if component_configer.memory_key:
            self.memory_key = component_configer.memory_key
        if component_configer.max_tokens:
            self.max_tokens = component_configer.max_tokens
        return self



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/22 10:27
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: test_embedding.py
import asyncio
import unittest

from agentuniverse.agent.action.knowledge.embedding.openai_embedding import OpenAIEmbedding


class EmbeddingTest(unittest.TestCase):
    """
    Test cases for Embedding class
    """

    def setUp(self) -> None:
        self.embedding = OpenAIEmbedding(embedding_model_name='text-embedding-3-small',
                                         dimensions=1536)

    def test_get_embeddings(self) -> None:
        res = self.embedding.get_embeddings(texts=["hello world"])
        print(res)

    def test_async_get_embeddings(self) -> None:
        res = asyncio.run(self.embedding.async_get_embeddings(texts=["hello world"]))
        print(res)

    def test_as_langchain(self) -> None:
        langchain_embedding = self.embedding.as_langchain()
        res = langchain_embedding.embed_documents(texts=["hello world"])
        print(res)


if __name__ == '__main__':
    unittest.main()



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/13 11:56
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: test_context_util.py

import queue
import time
import threading

import pytest

from agentuniverse.base.context.framework_context_manager import FrameworkContextManager
from agentuniverse.base.context.framework_context import FrameworkContext

context_manager: FrameworkContextManager = FrameworkContextManager()


def add(q: queue.Queue):
    with FrameworkContext({"add_value": 1}):
        for i in range(10):
            add_value = context_manager.get_context("add_value")
            add_value += 1
            context_manager.set_context("add_value", add_value)
            time.sleep(0.001)
        q.put(context_manager.get_context("add_value"))


async def async_add(q: queue.Queue):
    add(q)


@pytest.mark.asyncio
async def test_context_thread_and_async_isolation():
    queue1 = queue.Queue()
    queue2 = queue.Queue()
    queue3 = queue.Queue()
    queue4 = queue.Queue()
    t1 = threading.Thread(target=add, args=(queue1,))
    t2 = threading.Thread(target=add, args=(queue2,))
    t1.start()
    t2.start()
    t1.join()
    t2.join()
    await async_add(queue3)
    await async_add(queue4)
    assert queue1.get() == 11
    assert queue2.get() == 11
    assert queue3.get() == 11
    assert queue4.get() == 11


if __name__ == "__main__":
    pytest.main([__file__, "-s"])



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/18 10:56
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: prompt.py

introduction = """
你是一位精通信息分析的ai助手。
"""

target = """
你的目标是判断问题对应的答案是否提供了有价值的信息，并对问题的答案做出建议和评价。
"""

instruction = """
你的工作是对用户根据指定问题给出的答案内容，判断答案是否有用并且给出具体的评价和建议。

你需要遵守以下规则:
1.回答必须是完整回答了我的问题，才是有用的。
2.回答如果是给出了一种查询信息的方式，是无用的。
--------------------------------------------------------------------------------------------------------------------------
输出必须是按照以下格式化的Json代码片段，suggestion字段是判断这个回答对问题是否有用的思考过程，is_useful字段是判断这个回答对问题是否有用的结果。
{{
    "suggestion": string,
    "is_useful": true/false
}}

之前的对话:
{chat_history}

背景信息是:
{background}

开始!
指定的问题是: {input}
给出的答案是: {expressing_result}
"""



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/22 15:52
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: query.py

from typing import Optional, List

from pydantic import BaseModel, Field


class Query(BaseModel):
    """The basic class for the knowledge query.

    Attributes:
        query_str (Optional[str]): The query string.
        similarity_top_k (int): The number of top results to return.
        embedding (List[float]): The specific embedding data of the query.
    """

    query_str: Optional[str] = None
    similarity_top_k: int = 2
    embedding: List[float] = Field(default_factory=list)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/27 11:37
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: memory_util.py
from typing import List

from agentuniverse.agent.memory.message import Message


def generate_messages(memories: list) -> List[Message]:
    messages = []
    for memory in memories:
        message: Message = Message(type=memory.get('type'), content=memory.get('content'))
        messages.append(message)
    return messages



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/22 15:50
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: store.py
from typing import Any, List, Optional
from pydantic import BaseModel

from agentuniverse.agent.action.knowledge.embedding.embedding import Embedding
from agentuniverse.agent.action.knowledge.store.document import Document
from agentuniverse.agent.action.knowledge.store.query import Query


class Store(BaseModel):
    """The basic class for the knowledge store.

    Store of the knowledge, store class is used to store knowledge
    and provide retrieval capabilities,
    vector storage, such as ChromaDB store, or non-vector storage, such as Redis Store.

    Attributes:
        client (Any): The client of the store,
        client is usually used to connect to the storage and provides knowledge insertion,
        update, deletion, and query operations

        async_client (Any): The async client of the store,
        the function is the same as that of the `client` parameter  in asynchronous mode.

        embedding_model (Embedding): The embedding model of the store,
        used to provided embedding operations on texts to generate a list of floats.
    """

    client: Any = None
    async_client: Any = None
    embedding_model: Optional[Embedding] = None

    class Config:
        """Configuration for this pydantic object."""
        arbitrary_types_allowed = True

    def __init__(self, **kwargs):
        """Initialize the store class."""
        super().__init__(**kwargs)
        if self.client is None:
            self.client = self._new_client()
        if self.async_client is None:
            self.async_client = self._new_async_client()

    def _new_client(self) -> Any:
        """Initialize the client."""
        pass

    def _new_async_client(self) -> Any:
        """Initialize the async client."""
        pass

    def query(self, query: Query, **kwargs) -> List[Document]:
        """Query documents."""
        raise NotImplementedError

    async def async_query(self, query: Query, **kwargs) -> List[Document]:
        """Asynchronously query documents."""
        raise NotImplementedError

    def insert_documents(self, documents: List[Document], **kwargs):
        """Insert documents into the store."""
        raise NotImplementedError

    async def async_insert_documents(self, documents: List[Document], **kwargs):
        """Asynchronously insert documents into the store."""
        raise NotImplementedError

    def insert_document(self, document: Document, **kwargs):
        """Insert document into the store."""
        raise NotImplementedError

    async def async_insert_document(self, document: Document, **kwargs):
        """Asynchronously insert document into the store."""
        raise NotImplementedError

    def delete_document(self, document_id: str, **kwargs):
        """Delete the specific document by the document id."""
        raise NotImplementedError

    async def async_delete_document(self, document_id: str, **kwargs):
        """Asynchronously delete the specific document by the document id."""
        raise NotImplementedError



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/19 20:03
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: reviewing_agent.py
"""Reviewing Agent class."""
import json
from agentuniverse.agent.agent import Agent
from agentuniverse.agent.input_object import InputObject


class ReviewingAgent(Agent):
    """Reviewing Agent module."""

    def input_keys(self) -> list[str]:
        """Return the input keys of the Agent."""
        return ['input', 'expressing_result']

    def output_keys(self) -> list[str]:
        """Return the output keys of the Agent."""
        return ['output']

    def parse_input(self, input_object: InputObject, planner_input: dict) -> dict:
        """Planner parameter parsing.

        Args:
            input_object(InputObject): agent parameter object
            planner_input(dict): Planner input
        Returns:
            dict: Planner parameter.
        """
        planner_input['input'] = input_object.get_data('input')
        planner_input['expressing_result'] = input_object.get_data('expressing_result').get_data('output')
        return planner_input

    def parse_result(self, planner_result: dict) -> dict:
        """Planner result parser.

        Args:
            planner_result(dict): Planner result
        Returns:
            dict: Agent result object.
        """
        agent_result = dict()

        output = planner_result.get('output')
        output = json.loads(output)
        is_useful = output.get('is_useful')
        if is_useful is None:
            is_useful = False
        is_useful = bool(is_useful)
        if is_useful:
            score = 60
        else:
            score = 0

        agent_result['output'] = output
        agent_result['score'] = score
        agent_result['suggestion'] = output.get('suggestion')
        return agent_result



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/18 22:03
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: mock_simple_service.py

class SimpleService:
    """Mock class of agentuniverse.agent_serve.service.Service."""

    def __init__(self, service_name: str):
        self.service_code = SimpleService.generate_service_code(service_name)

    @staticmethod
    def generate_service_code(service_name: str) -> str:
        return "test_app.service." + service_name



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/4/2 16:18
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: langchain_instance.py
from typing import Any, List, Optional

from langchain.callbacks.manager import AsyncCallbackManagerForLLMRun, CallbackManagerForLLMRun
from langchain.chat_models import ChatOpenAI
from langchain_community.chat_models.openai import acompletion_with_retry
from langchain.schema import BaseMessage, ChatResult

from agentuniverse.llm.llm import LLM


class LangchainOpenAI(ChatOpenAI):
    """Langchain OpenAI LLM wrapper."""

    def __init__(self, llm: LLM):
        """The __init__ method.

        The AgentUniverse LLM instance is passed to this class as an argument.
        Convert the attributes of AFAF LLM instance to the LangchainOpenAI object for initialization

        Args:
            llm (LLM): the AFAF LLM instance.
        """
        init_params = dict()
        init_params['model_name'] = llm.model_name if llm.model_name is not None else 'gpt-3.5-turbo'
        init_params['temperature'] = llm.temperature if llm.temperature is not None else 0.7
        init_params['request_timeout'] = llm.request_timeout
        init_params['max_tokens'] = llm.max_tokens
        init_params['max_retries'] = llm.max_retries if llm.max_retries is not None else 2
        init_params['streaming'] = llm.streaming if llm.streaming is not None else False
        init_params['openai_api_key'] = llm.openai_api_key
        init_params['openai_organization'] = llm.openai_organization
        init_params['openai_api_base'] = llm.openai_api_base
        super().__init__(**init_params)

    def _generate(
            self,
            messages: List[BaseMessage],
            stop: Optional[List[str]] = None,
            run_manager: Optional[CallbackManagerForLLMRun] = None,
            stream: Optional[bool] = None,
            **kwargs,
    ) -> ChatResult:
        """Run the Langchain OpenAI LLM."""
        message_dicts, params = self._create_message_dicts(messages, stop)
        params = {**params, **kwargs}
        response = self.completion_with_retry(
            messages=message_dicts, run_manager=run_manager, **params
        )
        return self._create_chat_result(response)

    async def _agenerate(
            self,
            messages: List[BaseMessage],
            stop: Optional[List[str]] = None,
            run_manager: Optional[AsyncCallbackManagerForLLMRun] = None,
            stream: Optional[bool] = None,
            **kwargs: Any,
    ) -> ChatResult:
        """Asynchronously run the Langchain OpenAI LLM."""
        message_dicts, params = self._create_message_dicts(messages, stop)
        params = {**params, **kwargs}
        response = await acompletion_with_retry(
            self, messages=message_dicts, **params
        )
        return self._create_chat_result(response)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/14 18:02
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: prompt.py
from langchain_core.prompts import PromptTemplate

_DEFAULT_EN_SUMMARIZER_TEMPLATE = """Progressively summarize the lines of conversation provided,
 adding onto the previous summary returning a new summary.

EXAMPLE
Current summary:
The human asks what the AI thinks of artificial intelligence. The AI thinks artificial intelligence is a force for good.

New lines of conversation:
Human: Why do you think artificial intelligence is a force for good?
AI: Because artificial intelligence will help humans reach their full potential.

New summary:
The human asks what the AI thinks of artificial intelligence.
The AI thinks artificial intelligence is a force for good because it will help humans reach their full potential.
END OF EXAMPLE

Current summary:
{summary}

New lines of conversation:
{new_lines}

New summary:"""

_DEFAULT_CN_SUMMARIZER_TEMPLATE = """逐步总结所提供对话内容，基于先前的概要信息，返回新的概要。

例如：
当前概要:
人类询问AI对人工智能的看法。AI认为人工智能是一股积极的力量。

新一轮对话：
人类：你为什么认为人工智能是一股积极的力量？
AI：因为人工智能将帮助人类发挥他们的全部潜能。

新的概要:
人类询问AI对人工智能的看法。
AI认为人工智能是一股积极的力量，因为它将帮助人们发挥他们的全部潜能。

示例结束。

当前概要:
{summary}

新一轮对话:
{new_lines}

新的概要:"""

SUMMARY_PROMPT = PromptTemplate(
    input_variables=["summary", "new_lines"], template=_DEFAULT_EN_SUMMARIZER_TEMPLATE
)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/18 21:03
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: mock_application_config_manager.py

BASE_INFO_APPNAME = "test_app"


class MockAppConfiger:

    def __init__(self):
        self.base_info_appname = BASE_INFO_APPNAME


class MockApplicationConfigManager:
    """Mock class of ApplicationConfigManager."""

    def __init__(self):
        self.app_configer = MockAppConfiger()


# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/13 16:09
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: mock_log_client.py

from aliyun.log.putlogsrequest import PutLogsRequest


class LogClient:
    """Mock class of aliyun.log.logclient.LogClient."""

    def __init__(self,
                 end_point: str,
                 access_key_id,
                 access_key_secret):
        self.endpoint = end_point
        self.access_key_id = access_key_id
        self.access_key_secret = access_key_secret

    def put_logs(self,request: PutLogsRequest):
        return True



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/4/2 16:15
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: openai_llm.py
from typing import Any, Optional

from langchain_core.language_models.base import BaseLanguageModel
from openai import OpenAI, AsyncOpenAI
from pydantic import Field

from agentuniverse.llm.langchain_instance import LangchainOpenAI
from agentuniverse.llm.llm import LLM, LLMOutput
from agentuniverse.base.util.env_util import get_from_env


class OpenAILLM(LLM):
    """The openai llm class.

    Attributes:
        openai_api_key (Optional[str], optional): The API key for the OpenAI API.
        This automatically infers the `openai_api_key` from the environment variable `OPENAI_API_KEY` if not provided.

        openai_organization (Optional[str], optional): The OpenAI organization.
        This automatically infers the `openai_organization` from the environment variable `OPENAI_ORGANIZATION` if not provided.

        openai_api_base (Optional[str], optional): The OpenAI base url.
        This automatically infers the `openai_api_base` from the environment variable `OPENAI_API_BASE` if not provided.

        openai_client_args (Optional[dict], optional): Additional arguments to pass to the OpenAI client.
   """

    openai_api_key: Optional[str] = Field(default_factory=lambda: get_from_env("OPENAI_API_KEY"))
    openai_organization: Optional[str] = Field(default_factory=lambda: get_from_env("OPENAI_ORGANIZATION"))
    openai_api_base: Optional[str] = Field(default_factory=lambda: get_from_env("OPENAI_API_BASE"))
    openai_client_args: Optional[dict] = None

    def _new_client(self):
        """Initialize the openai client."""
        return OpenAI(
            api_key=self.openai_api_key,
            organization=self.openai_organization,
            base_url=self.openai_api_base,
            timeout=self.request_timeout,
            max_retries=self.max_retries,
            **(self.openai_client_args or {}),
        )

    def _new_async_client(self):
        """Initialize the openai async client."""
        return AsyncOpenAI(
            api_key=self.openai_api_key,
            organization=self.openai_organization,
            base_url=self.openai_api_base,
            timeout=self.request_timeout,
            max_retries=self.max_retries,
            **(self.openai_client_args or {}),
        )

    def call(self, messages: list, **kwargs: Any) -> LLMOutput:
        """Run the OpenAI LLM.

        Args:
            messages (list): The messages to send to the LLM.
            **kwargs: Arbitrary keyword arguments.
        """
        if self.client is None:
            self.client = self._new_client()
        chat_completion = self.client.chat.completions.create(
            model=self.model_name,
            messages=messages,
            max_tokens=self.max_tokens,
            stream=self.streaming,
            **kwargs,
        )
        text = chat_completion.choices[0].message.content
        return LLMOutput(text=text)

    async def acall(self, messages: list, **kwargs: Any) -> LLMOutput:
        """Asynchronously run the OpenAI LLM.

        Args:
            messages (list): The messages to send to the LLM.
            **kwargs: Arbitrary keyword arguments.
        """

        if self.async_client is None:
            self.async_client = self._new_async_client()
        chat_completion = await self.async_client.chat.completions.create(
            model=self.model_name,
            messages=messages,
            max_tokens=self.max_tokens,
            stream=self.streaming,
            **kwargs,
        )
        text = chat_completion.choices[0].message.content
        return LLMOutput(text=text)

    def as_langchain(self) -> BaseLanguageModel:
        """Convert the AFAF openai llm class to the langchain openai llm class."""
        return LangchainOpenAI(self)

    def set_by_agent_model(self, **kwargs) -> None:
        """ Assign values of parameters to the OpenAILLM model in the agent configuration."""
        super().set_by_agent_model(**kwargs)
        if 'openai_api_key' in kwargs and kwargs['openai_api_key']:
            self.openai_api_key = kwargs['openai_api_key']
        if 'openai_api_base' in kwargs and kwargs['openai_api_base']:
            self.openai_api_base = kwargs['openai_api_base']
        if 'openai_client_args' in kwargs and kwargs['openai_client_args']:
            self.openai_client_args = kwargs['openai_client_args']



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
import unittest

# @Time    : 2024/3/18 22:45
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: test_service_instance.py
import pytest
from unittest.mock import patch

from agentuniverse.agent_serve.service_instance import ServiceInstance
from tests.test_agentuniverse.mock.agent_serve.mock_service_manager import (
    ServiceManager, TEST_SERVICE_LIST, TEST_STR
)


@patch('agentuniverse.agent_serve.service_instance.ServiceManager', new=ServiceManager)
def test_service_instance():
    unittest.TestCase().assertRaises(Exception,
                                     ServiceInstance.__init__,
                                     "not_exist_service_code")

    service_instance = ServiceInstance(TEST_SERVICE_LIST[0])
    result = service_instance.run()
    assert result == TEST_STR


if __name__ == "__main__":
    pytest.main([__file__, "-s"])




from enum import Enum
from typing import List
from langchain.chains.summarize import load_summarize_chain
import asyncio
from langchain_core.documents import Document
from langchain_core.prompts import PromptTemplate
from agentuniverse.llm.llm import LLM
from agentuniverse.llm.llm_manager import LLMManager
from agentuniverse.prompt.prompt_model import AgentPromptModel


class PromptProcessEnum(Enum):
    TRUNCATE = 'truncate'
    STUFF = 'stuff'
    MAP_REDUCE = 'map_reduce'

    @classmethod
    def from_value(cls, value):
        for member in cls:
            if member.value.lower() == value.lower():
                return member
        raise ValueError(f"No enum member with value: {value}")


summarize_prompt_template = """
写下这段文字的详细摘要，输出结果中需要涵盖文本的要点、数据以及任何重要的细节。
注意：结果输出时必须用中文。
{text}
"""

combine_prompt_template = """
写出以下由三重反引号分隔的文本的详细摘要，输出结果中需要涵盖文本的要点、数据以及任何重要的细节。
注意：结果输出时必须用中文。
```{text}```
"""

summarize_prompt = PromptTemplate(template=summarize_prompt_template, input_variables=["text"])
combine_prompt = PromptTemplate(template=combine_prompt_template, input_variables=["text"])


def summarize_by_stuff(texts: List[str], llm: LLM):
    """
    stuff summarization -- general method
    """
    stuff_chain = load_summarize_chain(llm.as_langchain(), chain_type='stuff', verbose=True, prompt=summarize_prompt)
    return asyncio.run(stuff_chain.arun([Document(page_content=text) for text in texts]))


def summarize_by_map_reduce(texts: List[str], llm: LLM, agent_llm: LLM):
    """
    map reduce summarization -- general method
    """
    texts = split_texts(texts, agent_llm)
    map_reduce_chain = load_summarize_chain(llm.as_langchain(), chain_type='map_reduce', verbose=True,
                                            map_prompt=summarize_prompt,
                                            combine_prompt=combine_prompt)
    return asyncio.run(map_reduce_chain.arun([Document(page_content=text) for text in texts]))


def split_text_on_tokens(text: str, text_token, chunk_size=800, chunk_overlap=100) -> List[str]:
    """Split incoming text and return chunks using tokenizer."""
    # Calculate the number of characters represented by each token.
    char_per_token = len(text) / text_token
    chunk_char_size = int(chunk_size * char_per_token)
    chunk_char_overlap = int(chunk_overlap * char_per_token)

    result = []
    current_position = 0

    while current_position + chunk_char_overlap < len(text):
        if current_position + chunk_char_size >= len(text):
            chunk = text[current_position:]
        else:
            chunk = text[current_position:current_position + chunk_char_size]

        result.append(chunk)
        current_position += chunk_char_size - chunk_char_overlap

    return result


def split_texts(texts: list[str], agent_llm: LLM, chunk_size=800, chunk_overlap=100, retry=True) -> list[str]:
    """
    split texts into chunks with the fixed token length -- general method
    """
    try:
        split_texts_res = []
        llm = agent_llm.as_langchain()
        for text in texts:
            text_token = llm.get_num_tokens(text)
            split_texts_res.extend(
                split_text_on_tokens(text=text, text_token=text_token, chunk_size=chunk_size,
                                     chunk_overlap=chunk_overlap))
        return split_texts_res
    except Exception as e:
        if retry:
            return split_texts(texts=texts, agent_llm=agent_llm, retry=False)
        raise ValueError("split text failed, exception=" + str(e))


def truncate_content(content: str, token_length: int, agent_llm: LLM) -> str:
    """
    truncate the content based on the llm token limit
    """
    return str(split_texts(texts=[content], chunk_size=token_length, agent_llm=agent_llm)[0])


def generate_template(agent_prompt_model: AgentPromptModel, prompt_assemble_order: list[str]) -> str:
    """Convert the agent prompt model to an ordered list.

    Args:
        agent_prompt_model (AgentPromptModel): The agent prompt model.
        prompt_assemble_order (list[str]): The prompt assemble ordered list.
    Returns:
        list: The ordered list.
    """
    values = []
    for attr in prompt_assemble_order:
        value = getattr(agent_prompt_model, attr, None)
        if value is not None:
            values.append(value)

    return "\n".join(values)


def process_llm_token(prompt_template: PromptTemplate, profile: dict, planner_input: dict):
    """Process the prompt template based on the prompt processor.

    Args:
        prompt_template (PromptTemplate): The prompt template.
        profile (dict): The profile.
        planner_input (dict): The planner input.
    """
    llm_model: dict = profile.get('llm_model')
    llm_name: str = llm_model.get('name')

    prompt_processor: dict = llm_model.get('prompt_processor') or dict()
    prompt_processor_type: str = prompt_processor.get('type') or PromptProcessEnum.TRUNCATE.value
    prompt_processor_llm: str = prompt_processor.get('llm') or llm_name
    prompt_input_dict = {key: planner_input[key] for key in prompt_template.input_variables if key in planner_input}

    agent_llm: LLM = LLMManager().get_instance_obj(llm_name)
    llm_max_tokens: int = agent_llm.max_tokens
    prompt_llm: LLM = LLMManager().get_instance_obj(prompt_processor_llm)

    prompt = prompt_template.format(**prompt_input_dict)
    prompt_token_length: int = len(prompt)

    if prompt_token_length <= llm_max_tokens:
        return

    remaining_token_length: int = prompt_token_length - llm_max_tokens

    process_prompt_type_enum = PromptProcessEnum.from_value(prompt_processor_type)

    content = planner_input.get('background')
    if process_prompt_type_enum == PromptProcessEnum.TRUNCATE:
        planner_input['background'] = truncate_content(content, remaining_token_length, agent_llm)
    elif process_prompt_type_enum == PromptProcessEnum.STUFF:
        planner_input['background'] = summarize_by_stuff([content], prompt_llm)
    elif process_prompt_type_enum == PromptProcessEnum.MAP_REDUCE:
        planner_input['background'] = summarize_by_map_reduce([content], prompt_llm, agent_llm)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/12 19:36
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: agent.py
"""The definition of agent paradigm."""
from abc import abstractmethod
from typing import Optional

from agentuniverse.agent.agent_model import AgentModel
from agentuniverse.agent.input_object import InputObject
from agentuniverse.agent.output_object import OutputObject
from agentuniverse.agent.plan.planner.planner import Planner
from agentuniverse.agent.plan.planner.planner_manager import PlannerManager
from agentuniverse.base.component.component_base import ComponentBase
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.config.application_configer.application_config_manager \
    import ApplicationConfigManager
from agentuniverse.base.config.component_configer.configers.agent_configer \
    import AgentConfiger
from agentuniverse.llm.llm import LLM


class Agent(ComponentBase):
    """The parent class of all agent models, containing only attributes."""

    agent_model: Optional[AgentModel] = None

    def __init__(self):
        """Initialize the AgentModel with the given keyword arguments."""
        super().__init__(component_type=ComponentEnum.AGENT)

    @abstractmethod
    def input_keys(self) -> list[str]:
        """Return the input keys of the Agent."""
        pass

    @abstractmethod
    def output_keys(self) -> list[str]:
        """Return the output keys of the Agent."""
        pass

    @abstractmethod
    def parse_input(self, input_object: InputObject, planner_input: dict) -> dict:
        """Planner parameter parsing.

        Args:
            input_object(InputObject): agent parameter object
            planner_input(dict): Planner input
        Returns:
            dict: Planner parameter.
        """
        pass

    @abstractmethod
    def parse_result(self, planner_result: dict) -> dict:
        """Planner result parser.

        Args:
            planner_result(dict): Planner result
        Returns:
            dict: Agent result object.
        """
        pass

    def run(self, **kwargs) -> OutputObject:
        """Agent instance running entry.

        Returns:
            OutputObject: Agent execution result
        """
        self.input_check(kwargs)
        input_object = InputObject(kwargs)
        execute_result = self.execute(input_object)
        self.output_check(execute_result)
        output_object = OutputObject(execute_result)
        return output_object

    def execute(self, input_object: InputObject) -> dict:
        """Execute agent instance.

        Args:
            input_object(InputObject): agent parameter object
        Returns:
            dict: Agent result object.
        """
        planner_input = self.pre_parse_input(input_object)

        planner_base: Planner = PlannerManager().get_instance_obj(self.agent_model.plan.get('planner').get('name'))
        planner_result = planner_base.invoke(self.agent_model, planner_input, input_object)

        parser_result = self.parse_result(planner_result)
        return parser_result

    def pre_parse_input(self, input_object) -> dict:
        """Agent execution parameter pre-parsing.

        Args: input_object(InputObject): agent parameter object
        Returns:
            dict: Planner input
        """
        planner_input = dict()
        planner_input['chat_history'] = input_object.get_data('chat_history') or ''
        planner_input['background'] = input_object.get_data('background') or ''

        self.parse_input(input_object, planner_input)
        return planner_input

    def get_instance_code(self) -> str:
        """Return the full name of the agent."""
        appname = ApplicationConfigManager().app_configer.base_info_appname
        name = self.agent_model.info.get('name')
        return (f'{appname}.'
                f'{self.component_type.value.lower()}.{name}')

    def input_check(self, kwargs: dict):
        """Agent parameter check."""
        for key in self.input_keys():
            if key not in kwargs.keys():
                raise Exception(f'Input must have key: {key}.')

    def output_check(self, kwargs: dict):
        """Agent result check."""
        if not isinstance(kwargs, dict):
            raise Exception('Output type must be dict.')
        for key in self.output_keys():
            if key not in kwargs.keys():
                raise Exception(f'Output must have key: {key}.')

    def initialize_by_component_configer(self, component_configer: AgentConfiger) -> 'Agent':
        """Initialize the LLM by the ComponentConfiger object.

        Args:
            component_configer(LLMConfiger): the ComponentConfiger object
        Returns:
            LLM: the LLM object
        """
        agent_config: Optional[AgentConfiger] = component_configer.load()
        info: Optional[dict] = agent_config.info
        profile: Optional[dict] = agent_config.profile
        plan: Optional[dict] = agent_config.plan
        memory: Optional[dict] = agent_config.memory
        action: Optional[dict] = agent_config.action
        agent_model: Optional[AgentModel] = AgentModel(info=info, profile=profile,
                                                       plan=plan, memory=memory, action=action)
        self.agent_model = agent_model
        return self



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/4/2 15:27
# @Author  : jerry.zzw 
# @Email   : jerry.zzw@antgroup.com
# @FileName: agentuniverse.py
import importlib
import sys
from pathlib import Path

from agentuniverse.base.annotation.singleton import singleton
from agentuniverse.base.component.application_component_manager import ApplicationComponentManager
from agentuniverse.base.component.component_base import ComponentBase
from agentuniverse.base.config.application_configer.app_configer import AppConfiger
from agentuniverse.base.config.application_configer.application_config_manager import ApplicationConfigManager
from agentuniverse.base.config.component_configer.component_configer import ComponentConfiger
from agentuniverse.base.component.component_configer_util import ComponentConfigerUtil
from agentuniverse.base.config.config_type_enum import ConfigTypeEnum
from agentuniverse.base.config.configer import Configer
from agentuniverse.base.config.custom_configer.custom_key_configer import CustomKeyConfiger
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.util.system_util import get_project_root_path
from agentuniverse.base.util.logging.logging_util import init_loggers
from agentuniverse.agent_serve.web.request_task import RequestLibrary
from agentuniverse.agent_serve.web.web_booster import GunicornApplication


@singleton
class AgentUniverse(object):
    """AgentUniverse framework object, responsible for the framework initialization,
       system variables management, etc."""

    def __init__(self):
        self.__application_container = ApplicationComponentManager()
        self.__config_container: ApplicationConfigManager = ApplicationConfigManager()
        self.__system_default_package = ['agentuniverse']

    def start(self, config_path: str = None):
        """Start the AgentUniverse framework."""
        # step0: get default config path
        project_root_path = get_project_root_path()
        app_path = project_root_path / 'app'
        if app_path.exists():
            sys.path.append(str(app_path))
        if not config_path:
            config_path = project_root_path / 'config' / 'config.toml'
            config_path = str(config_path)

        # step1: load the configuration file
        configer = Configer(path=config_path).load()
        app_configer = AppConfiger().load_by_configer(configer)
        self.__config_container.app_configer = app_configer

        # Load User custom key.
        custom_key_configer_path = self.__parse_sub_config_path(
            configer.value.get('SUB_CONFIG_PATH', {}).get('custom_key_path'),
            config_path)
        CustomKeyConfiger(custom_key_configer_path)

        # Init loguru loggers.
        log_config_path = self.__parse_sub_config_path(
            configer.value.get('SUB_CONFIG_PATH', {}).get('log_config_path'),
            config_path)
        init_loggers(log_config_path)

        # Init web request task database.
        RequestLibrary(configer=configer)

        # Init gunicorn web server.
        gunicorn_config_path = self.__parse_sub_config_path(
            configer.value.get('SUB_CONFIG_PATH', {})
            .get('gunicorn_config_path'), config_path)
        GunicornApplication(config_path=gunicorn_config_path)

        # step2: scan and register the components
        self.__scan_and_register(self.__config_container.app_configer)

    def __scan_and_register(self, app_configer: AppConfiger):
        """Scan the component directory and register the components.

        Args:
            app_configer(AppConfiger): the AppConfiger object
        """
        core_agent_package_list = app_configer.core_agent_package_list or app_configer.core_default_package_list + self.__system_default_package
        core_knowledge_package_list = app_configer.core_knowledge_package_list or app_configer.core_default_package_list + self.__system_default_package
        core_llm_package_list = app_configer.core_llm_package_list or app_configer.core_default_package_list + self.__system_default_package
        core_planner_package_list = app_configer.core_planner_package_list or app_configer.core_default_package_list + self.__system_default_package
        core_tool_package_list = app_configer.core_tool_package_list or app_configer.core_default_package_list + self.__system_default_package
        core_service_package_list = app_configer.core_service_package_list or app_configer.core_default_package_list + self.__system_default_package
        core_memory_package_list = app_configer.core_memory_package_list or app_configer.core_default_package_list + self.__system_default_package

        component_package_map = {
            ComponentEnum.AGENT: core_agent_package_list,
            ComponentEnum.KNOWLEDGE: core_knowledge_package_list,
            ComponentEnum.LLM: core_llm_package_list,
            ComponentEnum.PLANNER: core_planner_package_list,
            ComponentEnum.TOOL: core_tool_package_list,
            ComponentEnum.SERVICE: core_service_package_list,
            ComponentEnum.MEMORY: core_memory_package_list
        }

        component_configer_list_map = {}
        for component_enum, package_list in component_package_map.items():
            if not package_list:
                continue
            component_configer_list = self.__scan(package_list, ConfigTypeEnum.YAML, component_enum)
            component_configer_list_map[component_enum] = component_configer_list

        for component_enum, component_configer_list in component_configer_list_map.items():
            self.__register(component_enum, component_configer_list)

    def __scan(self,
               package_list: [str],
               config_type_enum: ConfigTypeEnum,
               component_enum: ComponentEnum) -> list:
        """Scan the component directory and return certain component configer list.

        Args:
            package_list(list): the package list
            config_type_enum(ConfigTypeEnum): the configuration file type enumeration
            component_enum(ComponentEnum): the component enumeration

        Returns:
            list: the component configer list
        """
        component_configer_list = []
        for package_name in package_list:
            package_path = self.__package_name_to_path(package_name)
            path = Path(package_path)
            config_files = path.rglob(f'*.{config_type_enum.value}')
            for config_file in config_files:
                config_file_str = str(config_file)
                configer = Configer(path=config_file_str).load()
                component_configer = ComponentConfiger().load_by_configer(configer)
                component_config_type = component_configer.get_component_config_type()
                if component_config_type == component_enum.value:
                    component_configer_list.append(component_configer)
        return component_configer_list

    def __register(self, component_enum: ComponentEnum, component_configer_list: list[ComponentConfiger]):
        """Register the components.

        Args:
            component_enum(ComponentEnum): the component enumeration
            component_configer_list(list): the component configer list
        """
        component_manager_clz = ComponentConfigerUtil.get_component_manager_clz_by_type(component_enum)
        for component_configer in component_configer_list:
            configer_clz = ComponentConfigerUtil.get_component_config_clz_by_type(component_enum)
            configer_instance: ComponentConfiger = configer_clz().load_by_configer(component_configer.configer)
            component_clz = ComponentConfigerUtil.get_component_object_clz_by_component_configer(configer_instance)
            component_instance: ComponentBase = component_clz().initialize_by_component_configer(configer_instance)
            if component_instance is None:
                continue
            component_manager_clz().register(component_instance.get_instance_code(), component_instance)

    def __package_name_to_path(self, package_name: str) -> str:
        """Convert the package name to the package path.

        Args:
            package_name(str): the package name

        Returns:
            str: the package path
        """
        # get the package spec
        spec = importlib.util.find_spec(package_name)
        if spec is None:
            raise ImportError(f"Can not find {package_name}")
        # get the package path
        package_path = spec.submodule_search_locations[0] if spec.submodule_search_locations else spec.origin
        return package_path

    def __parse_sub_config_path(self, input_path: str,
                                reference_file_path: str) -> str | None:
        """Resolve a sub config file path according to main config file.

            Args:
                input_path(str): Absolute or relative path of sub config file.
                reference_file_path(str): Main config file path.
            Returns:
                str or None: Final
        """
        if not input_path:
            return None

        input_path_obj = Path(input_path)
        if input_path_obj.is_absolute():
            combined_path = input_path_obj
        else:
            reference_file_path_obj = Path(reference_file_path)
            combined_path = reference_file_path_obj.parent / input_path_obj

        return str(combined_path)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/25 15:03
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: rag_agent.py
"""Planning agent module."""
from agentuniverse.agent.agent import Agent
from agentuniverse.agent.input_object import InputObject


class RagAgent(Agent):
    """Rag Agent class."""

    def input_keys(self) -> list[str]:
        """Return the input keys of the Agent."""
        return ['input']

    def output_keys(self) -> list[str]:
        """Return the output keys of the Agent."""
        return ['output']

    def parse_input(self, input_object: InputObject, planner_input: dict) -> dict:
        """Planner parameter parsing.

        Args:
            input_object(InputObject): agent parameter object
            planner_input(dict): Planner input
        Returns:
            dict: Planner input
        """
        input = input_object.get_data('input')
        planner_input['input'] = input
        return planner_input

    def parse_result(self, planner_result: dict) -> dict:
        """Planner result parser.

        Args:
            planner_result(dict): Planner result
        Returns:
            dict: Agent result object.
        """
        return planner_result



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/18 19:18
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: embedding.py
from abc import abstractmethod
from typing import List, Optional

from pydantic import BaseModel
from langchain_core.embeddings import Embeddings as LCEmbeddings


class Embedding(BaseModel):
    """The basic class for embedding.

    Attributes:
        embedding_model_name (Optional[str]): The name of the embedding model.
    """

    embedding_model_name: Optional[str] = None

    @abstractmethod
    def get_embeddings(self, text: List[str]) -> List[List[float]]:
        """Get embeddings."""

    @abstractmethod
    async def async_get_embeddings(self, texts: List[str]) -> List[List[float]]:
        """Asynchronously get embeddings."""

    def as_langchain(self) -> LCEmbeddings:
        """Convert the AFAF embedding class to the langchain embedding class."""
        pass



introduction = """
你是一位精通信息分析的ai助手。
"""

target = """
你的目标是对需要回答的问题进行拆解，生成多个子问题。
"""

instruction = """
根据需要回答的问题，给出一个逻辑递进的思考链路，帮助用户逐渐掌握知识并最终回答问题。
思考链路以子问题的形式体现，每个子问题都是一个完整的句子。
思考链路必须严格遵循需要回答的问题，不可以延伸问题，也不可以直接回答问题。
这个思考链路的每一步必须是简单、单一的。
复杂问题必须拆解成多个步骤。
每一步的问题必须是有答案的，不能是开放性的问题。
每一步的问题必须是完整的句子，不能有任何歧义。
请将这个问题拆解为多步，每一步不可以和原始问题相同，一步一步来。

输出必须是按照以下格式化的Json代码片段，thought字段代表拆解问题的思考过程，framework字段代表拆解后的子问题列表。
{{
    "thought": string,
    "framework": list
}}

之前的对话:
{chat_history}

背景信息是:
{background}

开始!

需要回答的问题是: {input}
"""



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/4/2 18:02
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: mock_agent.py

import json


class MockOutPut:
    def __init__(self, output: dict):
        self.__output = output

    def to_json_str(self) -> str:
        return json.dumps(self.__output, ensure_ascii=False)


class MockAgent:
    def __init__(self, run_result: dict):
        self.__run_result = MockOutPut(run_result)

    def run(self, **kwargs):
        return self.__run_result



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/20 10:10
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: test_knowledge.py
import unittest

from agentuniverse.agent.action.knowledge.embedding.openai_embedding import OpenAIEmbedding
from agentuniverse.agent.action.knowledge.knowledge import Knowledge
from agentuniverse.agent.action.knowledge.store.chroma_store import ChromaStore
from agentuniverse.agent.action.knowledge.store.query import Query
from agentuniverse.agent.action.knowledge.store.document import Document


class KnowledgeTest(unittest.TestCase):
    """
    Test cases for Knowledge class
    """

    def setUp(self) -> None:
        init_params = {}
        init_params['name'] = 'test_knowledge'
        init_params['description'] = 'test_knowledge_description'
        init_params['store'] = ChromaStore(collection_name="test_knowledge", embedding_model=OpenAIEmbedding(
            embedding_model_name='text-embedding-ada-002'))
        self.knowledge = Knowledge(**init_params)

    def test_store_insert_documents(self) -> None:
        store = self.knowledge.store
        store.insert_documents([Document(text='This is a document about engineer'),
                                Document(text='This is a document about finance')])

    def test_query(self) -> None:
        store = self.knowledge.store
        query = Query(query_str='Which stock is the best?', similarity_top_k=1)
        res = store.query(query)
        print(res)


if __name__ == '__main__':
    unittest.main()



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/13 15:22
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: prompt_base.py
"""Prompt base module."""
import re
from typing import Optional

from langchain_core.prompts import PromptTemplate

from agentuniverse.base.component.component_base import ComponentBase
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.util.prompt_util import generate_template
from agentuniverse.prompt.prompt_model import AgentPromptModel


class Prompt(ComponentBase):
    """Prompt class."""

    prompt_template: Optional[str] = None
    input_variables: Optional[list[str]] = None
    component_type: ComponentEnum = ComponentEnum.DEFAULT

    def as_langchain(self) -> PromptTemplate:
        """Convert the prompt template into a LangChain prompt template.

        Returns:
            PromptTemplate: The prompt template.
        """
        return PromptTemplate(template=self.prompt_template,
                              input_variables=self.input_variables)

    def build_prompt_template(self, user_agent_prompt_model: AgentPromptModel,
                              system_agent_prompt_model: AgentPromptModel,
                              prompt_assemble_order: list[str]):
        """Build the prompt template.

        Args:
            user_agent_prompt_model (AgentPromptModel): The user agent prompt model.
            system_agent_prompt_model (AgentPromptModel): The system agent prompt model.
            prompt_assemble_order (list[str]): The prompt assemble ordered list.

        Returns:
            PromptTemplate: The prompt template.
        """
        agent_prompt_model = user_agent_prompt_model + system_agent_prompt_model
        self.prompt_template = generate_template(agent_prompt_model, prompt_assemble_order)
        self.input_variables = re.findall(r'\{(.*?)}', self.prompt_template)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/13 15:50
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: test_logging_util.py

import os
import shutil
import time
import pytest

from agentuniverse.base.util.logging.logging_config import LoggingConfig
from agentuniverse.base.util.logging.logging_util import LOGGER, init_loggers, LOG_FILE_PREFIX
from agentuniverse.base.context.framework_context import FrameworkContext

test_context = {"LOG_CONTEXT": {"REQUEST_ID": "1111-2222-3333",
                                "AGENT_ID": "TEST_AGENT"}}


def test_logging_util():
    LoggingConfig.log_path = "./.test_log_dir"

    init_loggers()
    test_log_path = LoggingConfig.log_path
    with FrameworkContext(test_context):
        test_logger = LOGGER
        test_logger.info("test info log")
        test_logger.error("test error log")
        time.sleep(0.01)
        with open(os.path.join(test_log_path, LOG_FILE_PREFIX+"_all.log"), 'r') as file:
            first_line = file.readline().strip()
            second_line = file.readline().strip()
            assert "test info log" in first_line
            assert "test error log" in second_line
        with open(os.path.join(test_log_path, LOG_FILE_PREFIX+"_error.log"), 'r') as file:
            first_line = file.readline().strip()
            assert "test error log" in first_line

        # delete test logs
        try:
            shutil.rmtree(test_log_path)
            print(f"The directory {test_log_path} has been deleted.")
        except OSError as e:
            print(f"Error: {e.strerror}")


if __name__ == "__main__":
    pytest.main([__file__, "-s"])



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/22 15:44
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: knowledge.py
from typing import Optional, Dict, List, Any

from agentuniverse.agent.action.knowledge.reader.reader import Reader
from agentuniverse.agent.action.knowledge.store.document import Document
from agentuniverse.agent.action.knowledge.store.query import Query
from agentuniverse.agent.action.knowledge.store.store import Store
from agentuniverse.base.component.component_base import ComponentBase
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.config.application_configer.application_config_manager import ApplicationConfigManager
from agentuniverse.base.config.component_configer.configers.knowledge_configer import KnowledgeConfiger


class Knowledge(ComponentBase):
    """The basic class for knowledge model.

    Attributes:
        name (str): The name of the knowledge.

        description (str): The description of the knowledge.

        store (Store): The store of the knowledge, which is used to store knowledge
            and provide retrieval capabilities, such as ChromaDB store or Redis Store.

        reader (Reader): The reader of the knowledge, which is used to load data and generate knowledge.

        ext_info (Optional[Dict]): The extended information of the knowledge.
    """

    name: str = ""
    description: Optional[str] = None
    store: Store = None
    reader: Reader = None
    ext_info: Optional[Dict] = None

    def __init__(self, **kwargs):
        super().__init__(component_type=ComponentEnum.KNOWLEDGE, **kwargs)

    def insert_knowledge(self, **kwargs) -> None:
        """Insert the knowledge.

        Load data by the reader and insert the documents into the store.
        """
        document_list: List[Document] = self.reader.load_data()
        self.store.insert_documents(document_list, **kwargs)

    def query_knowledge(self, **kwargs) -> List[Document]:
        """Query the knowledge.

        Query documents from the store and return the results.
        """
        query = Query(**kwargs)
        return self.store.query(query)

    def get_instance_code(self) -> str:
        """Return the full name of the knowledge."""
        appname = ApplicationConfigManager().app_configer.base_info_appname
        return f'{appname}.{self.component_type.value.lower()}.{self.name}'

    def initialize_by_component_configer(self, component_configer: KnowledgeConfiger) -> 'Knowledge':
        """Initialize the Knowledge by the ComponentConfiger object.
        Args:
            component_configer(KnowledgeConfiger): the ComponentConfiger object
        Returns:
            Knowledge: the Knowledge object
        """
        if component_configer.name:
            self.name = component_configer.name
        if component_configer.description:
            self.description = component_configer.description
        if component_configer.ext_info:
            self.ext_info = component_configer.ext_info
        return self



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/18 10:46
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: prompt.py
introduction = """
你是一位精通信息分析的ai助手。
"""

target = """
你的目标是根据用户提供的问题，对查找到的知识进行整合、修正，用来回答用户问题。
"""

instruction = """
你需要仅仅根据我提供的背景知识来回答我的问题。
你的回答需要尽可能的详细、全面，包含足够多的数据信息。
--------------------------------------------------------------
需要遵守下面的规则：
1.去除重复的信息。
2.去除对回答问题没有帮助的信息。
3.去除错误的信息。
4.只可以使用这些信息来回答问题。
5.简明扼要，重点突出，不过多花哨词藻。
6.不在多处讲到同一个细节点，一个点只出现一次。
7.尽量多的使用数值类信息。
8.不可以出现XXX、ABC等不明确的词语。

之前的对话:
{chat_history}

背景信息是:
{background}

开始!
我的问题是: {input}

--------------------------------------------------------------
必须使用中文进行详细的回答。你需要聚焦在我的问题上，不要延伸这个问题。
让我们一步一步来。
"""



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
import unittest

# @Time    : 2024/3/18 22:06
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: test_service_manager.py
import pytest
from unittest.mock import patch, MagicMock

from agentuniverse.agent_serve.service_manager import ServiceManager
from tests.test_agentuniverse.mock.agent_serve.mock_simple_service import SimpleService
from tests.test_agentuniverse.mock.agent_serve.mock_application_config_manager import MockApplicationConfigManager

@patch("agentuniverse.agent_serve.service_manager.Service", new=SimpleService)
@patch("agentuniverse.base.component.component_manager_base.ApplicationConfigManager", new=MockApplicationConfigManager)
def test_service_manager():
    service_manager: ServiceManager = ServiceManager()
    mock_service = MagicMock()
    mock_service.name = "test_service_1"
    mock_service.service_code = SimpleService.generate_service_code(
        mock_service.name
    )
    service_manager.register(mock_service.service_code, mock_service)

    service = service_manager.get_instance_obj(mock_service.name)
    assert service.name == mock_service.name

    # register a service with same name
    unittest.TestCase().assertRaises(
        ValueError,
        service_manager.register,
        mock_service.service_code,
        mock_service
    )

    assert len(service_manager.get_instance_name_list()) == 1
    service_manager.unregister(
        mock_service.service_code
    )
    assert len(service_manager.get_instance_name_list()) == 0


if __name__ == "__main__":
    pytest.main([__file__, "-s"])



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/22 16:31
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: chroma_store.py
from typing import List, Any, Optional

import chromadb
from chromadb import QueryResult
from chromadb.api.models import Collection

from agentuniverse.agent.action.knowledge.store.document import Document
from agentuniverse.agent.action.knowledge.store.query import Query
from agentuniverse.agent.action.knowledge.store.store import Store


class ChromaStore(Store):
    """Object encapsulating the ChromaDB store that has vector search enabled.

    The ChromaStore object provides insert and retrieval capabilities.

    Attributes:
        collection_name (str): The name of the chroma collection to use.
        collection (Collection): A chroma collection object.
        persist_path (Optional[str]): Path to save the chroma database.
    """

    collection_name: Optional[str] = 'chroma_db'
    collection: Collection = None
    persist_path: Optional[str] = None

    def __init__(self, **kwargs):
        """Initialize the chroma store class."""
        super().__init__(**kwargs)
        if self.collection is None:
            # default to create a new collection or get an existed collection.
            self.collection = self.client.get_or_create_collection(name=self.collection_name)

    def _new_client(self) -> Any:
        """Initialize the chroma client."""
        if self.persist_path is None:
            return chromadb.PersistentClient()
        else:
            return chromadb.PersistentClient(path=self.persist_path)

    def query(self, query: Query, **kwargs) -> List[Document]:
        """Query the chroma collection with the given query and return the top k results.

        Args:
            query (Query): The query object.
            **kwargs: Arbitrary keyword arguments.

        Note:
            If there is no embedding in the specific query, but the embedding model is configured in the store,
            the embedding data of the query is automatically obtained by the embedding model.

        Returns:
            List[Document]: List of documents retrieved by the query.
        """

        embedding = query.embedding
        if self.embedding_model is not None and len(embedding) == 0:
            embedding = self.embedding_model.get_embeddings([query.query_str])[0]
        if len(embedding) > 0:
            query_result = self.collection.query(
                n_results=query.similarity_top_k,
                query_embeddings=[embedding]
            )
        else:
            query_result = self.collection.query(
                n_results=query.similarity_top_k,
                query_texts=[query.query_str]
            )
        # convert to the AFAF document format
        return self.to_documents(query_result)

    def insert_documents(self, documents: List[Document], **kwargs: Any):
        """Insert documents to the chroma collection.

        Args:
            documents (List[Document]): The documents to be inserted.
            **kwargs: Arbitrary keyword arguments.

        Note:
            If there is no embedding in the specific document, but the embedding model is configured in the store,
            the embedding data of the document is automatically obtained by the embedding model.
        """

        for document in documents:
            embedding = document.embedding
            if self.embedding_model is not None and len(embedding) == 0:
                embedding = self.embedding_model.get_embeddings([document.text])[0]
            self.collection.add(
                documents=[document.text],
                metadatas=[document.metadata],
                embeddings=[embedding] if embedding is not None else None,
                ids=[document.id]
            )

    @staticmethod
    def to_documents(query_result: QueryResult) -> List[Document]:
        """Convert the query results of ChromaDB to the AFAF document format."""

        if query_result is None:
            return []
        documents = []
        for i in range(len(query_result['ids'][0])):
            documents.append(Document(id=query_result['ids'][0][i],
                                      text=query_result['documents'][0][i],
                                      embedding=query_result['embeddings'][0][i]
                                      if query_result['embeddings'] is not None else [],
                                      metadata=query_result['metadatas'][0][i]
                                      if query_result['metadatas'] is not None else None))
        return documents



introduction = """
你是一位精通信息分析的ai助手。
"""

target = """
你的目标是根据用户的问题以及给出的背景信息给出答案。
"""

instruction = """
你需要遵守的规则是:
1. 必须使用中文回答用户提出的问题。
2. 答案的字数控制在200字以内。
3. 不采用background中的错误信息。
4. 要考虑答案和问题的相关性，不做对问题没有帮助的回答。
5. 简明扼要，重点突出，从不过多花哨词藻。
6. 不说模糊的推测。
7. 尽量多的使用数值类信息。

之前的对话:
{chat_history}

背景信息是:
{background}

开始!

需要回答的问题是: {input}
"""



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/14 10:42
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: system_util.py

from pathlib import Path

PROJECT_ROOT_PATH = None


def get_project_root_path() -> Path:
    """Get the project root path."""
    global PROJECT_ROOT_PATH
    if PROJECT_ROOT_PATH:
        return PROJECT_ROOT_PATH
    current_work_directory = Path.cwd()
    if current_work_directory.name != 'bootstrap':
        print(f"Warn: Boot file is not located under directory 'bootstrap', "
              f"but under '{current_work_directory.name}'")

    root_path = current_work_directory.parents[1]
    PROJECT_ROOT_PATH = root_path
    return root_path



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/25 11:18
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: request_task.py

import enum
from enum import Enum
import json
import queue
import time
import uuid
from datetime import datetime, timedelta
from threading import Thread
from typing import Optional

from .dal.request_library import RequestLibrary
from .dal.entity.request_do import RequestDO
from .thread_with_result import ThreadWithReturnValue
from agentuniverse.base.util.logging.logging_util import LOGGER

EOF_SIGNAL = '{"type": "EOF"}'


@enum.unique
class TaskStateEnum(Enum):
    """All possible state of a web request task."""
    INIT = "init"
    RUNNING = "running"
    FINISHED = "finished"
    FAIL = "fail"
    CANCELED = "canceled"


# All valid transitions of request task.
VALID_TRANSITIONS = {
    (TaskStateEnum.INIT, TaskStateEnum.RUNNING),
    (TaskStateEnum.INIT, TaskStateEnum.FAIL),
    (TaskStateEnum.INIT, TaskStateEnum.CANCELED),
    (TaskStateEnum.RUNNING, TaskStateEnum.FINISHED),
    (TaskStateEnum.RUNNING, TaskStateEnum.FAIL),
    (TaskStateEnum.RUNNING, TaskStateEnum.CANCELED),
}


class RequestTask:
    """The class that manages the agent service request, including state
    transitions and different types of execution flows."""

    def __init__(self, func, saved=True, **kwargs):
        """Init a RequestTask."""
        self.func: callable = func
        self.kwargs = kwargs
        self.request_id = uuid.uuid4().hex
        self.queue = queue.Queue(maxsize=100)
        self.thread: Optional[ThreadWithReturnValue] = None
        self.state = TaskStateEnum.INIT.value
        # Whether save to Database.
        self.saved = saved
        self.__request_do__ = self.add_request_do()

    def receive_steps(self):
        """Yield the stream data by getting data from the queue."""
        while True:
            output: str = self.queue.get()
            if output is None:
                break
            if output == EOF_SIGNAL:
                break
            yield "data:" + json.dumps({"process": output},
                                       ensure_ascii=False) + "\n\n"
        if self.canceled():
            return
        yield "data:" + json.dumps({"result": self.thread.result()},
                                   ensure_ascii=False) + "\n\n"

    def append_steps(self):
        """Tracing async service running state and update it to database."""
        try:
            self.next_state(TaskStateEnum.RUNNING)
            while True:
                output: str = self.queue.get()
                if output is None:
                    break
                if output == EOF_SIGNAL:
                    break
                if output != "" and output != " ":
                    self.__request_do__.steps.append(output)
                if self.saved:
                    RequestLibrary().update_request(self.__request_do__)
            if self.canceled():
                self.__request_do__.result['result'] = {
                    "result": "The task's tracking status has been canceled."}
            else:
                self.__request_do__.result['result'] = self.thread.result()
                self.next_state(TaskStateEnum.FINISHED)
            if self.saved:
                RequestLibrary().update_request(self.__request_do__)
        except Exception as e:
            LOGGER.error("request task update request state Fail: " + str(e))
            self.__request_do__.result['result'] = {"error_msg": str(e)}
            self.next_state(TaskStateEnum.FAIL)
            if self.saved:
                RequestLibrary().update_request(self.__request_do__)

    def async_run(self):
        """Run the service in async mode."""
        self.kwargs['output_stream'] = self.queue
        self.thread = ThreadWithReturnValue(target=self.func,
                                            kwargs=self.kwargs)
        self.thread.start()
        Thread(target=self.append_steps).start()
        Thread(target=self.check_state).start()

    def stream_run(self):
        """Run the service in a separate thread and yield result stream."""
        self.kwargs['output_stream'] = self.queue
        self.thread = ThreadWithReturnValue(target=self.func,
                                            kwargs=self.kwargs)
        self.thread.start()
        return self.receive_steps()

    def run(self):
        """Run the service synchronous and return the result."""
        self.next_state(TaskStateEnum.RUNNING)
        try:
            result = self.func(**self.kwargs)
            self.next_state(TaskStateEnum.FINISHED)
            self.__request_do__.result = {"result": result}
            return result
        except Exception as e:
            self.next_state(TaskStateEnum.FAIL)
            self.__request_do__.additional_args['error_msg'] = str(e)
            raise e
        finally:
            if self.saved:
                RequestLibrary().update_request(self.__request_do__)

    def next_state(self, next_state: TaskStateEnum):
        """Update request task state if the transition is valid."""
        if ((TaskStateEnum[self.__request_do__.state.upper()], next_state)
                in VALID_TRANSITIONS):
            self.__request_do__.state = next_state.value
        else:
            raise Exception("Invalid state transition")


    def check_state(self):
        """Keep check request task thread state every minute, if the thread
        is alive, update the request modified time in database."""
        while True:
            if self.thread is not None and self.thread.is_alive():
                LOGGER.debug(
                    "request:" + str(self.request_id) + "task thread alive")
                if self.saved:
                    RequestLibrary().update_gmt_modified(self.request_id)
                time.sleep(60)
                continue
            elif self.__request_do__.state == TaskStateEnum.RUNNING.value:
                # Waiting one minute to avoid skipping the state change step.
                time.sleep(60)
                if self.__request_do__.state == TaskStateEnum.RUNNING.value:
                    LOGGER.debug("request:" + str(self.request_id) +
                                 " task thread stop but state not end")
                    self.__request_do__.state = TaskStateEnum.FAIL.value
                    if self.saved:
                        RequestLibrary().update_request(
                            self.__request_do__)
            break

    def add_request_do(self):
        query_keys = ['question', 'query_content', 'query', 'request', 'input']
        query = next((self.kwargs[key] for key in query_keys if
                      self.kwargs.get(key) is not None),
                     "No relevant query was retrieved.")

        request_do = RequestDO(
            request_id=self.request_id,
            session_id="",
            query=query,
            state=TaskStateEnum.INIT.value,
            result=dict(),
            steps=[],
            additional_args=dict(),
            gmt_create=int(time.time()),
            gmt_modified=int(time.time()),
        )
        if self.saved:
            RequestLibrary().add_request(request_do)
        return request_do

    def result(self):
        """Get the result from service running thread."""
        return self.thread.result()

    @staticmethod
    def is_validate(request_do: RequestDO):
        """If there is no update within ten minutes and the status is neither
        completed nor failed, the task is considered to have failed."""
        if (request_do.gmt_modified < datetime.now() - timedelta(minutes=10)
                and request_do.state != TaskStateEnum.FINISHED.value
                and request_do.state != TaskStateEnum.FAIL.value):
            LOGGER.error("request task is validate fail" + str(request_do))
            request_do.state = TaskStateEnum.FAIL.value
            RequestLibrary().update_request(request_do)

    def cancel(self):
        """Cancel the request task. If a response SSE stream is working, put
        the EOF into the queue."""
        self.next_state(TaskStateEnum.CANCELED)
        if self.queue is not None:
            self.queue.put_nowait(EOF_SIGNAL)

    def request_state(self):
        """Return the request task state."""
        return self.__request_do__.state

    def canceled(self):
        """Whether task is canceled state."""
        return self.__request_do__.state == TaskStateEnum.CANCELED.value

    def finished(self):
        """Set task to finished state."""
        self.__request_do__.state = TaskStateEnum.FINISHED.value

    @staticmethod
    def query_request_state(request_id: str) -> dict:
        """Query the request data in database by given request_id.

        Args:
            request_id(str): Unique request id.
        """
        request_do = RequestLibrary().query_request_by_request_id(
            request_id)
        RequestTask.is_validate(request_do)
        if request_do is None:
            return {"state": TaskStateEnum.INIT.value,
                    "result": None,
                    "steps": None}
        return {
            "state": request_do.state,
            "result": request_do.result.get('result'),
            "steps": request_do.steps
        }



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/18 22:53
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: mock_service_manager.py
from unittest.mock import MagicMock

TEST_STR = "TEST_STR"
TEST_MOCK_SERVICE = MagicMock()
TEST_MOCK_SERVICE.run.return_value = TEST_STR
TEST_SERVICE_LIST = ["test_app.service.test_service"]
TEST_SERVICE_MAP = {"test_app.service.test_service":TEST_MOCK_SERVICE}


class ServiceManager:
    """Mock class of agentuniverse.agent_serve.service.Service."""

    def __init__(self):
        self.__service_list: list = TEST_SERVICE_LIST
        self.__service_map: dict = TEST_SERVICE_MAP

    def get_instance_obj(self, service_code: str):
        service_base = self.__service_map.get(service_code)
        if service_base is None:
            raise Exception(f"Service {service_code} not found.")
        return service_base



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/18 15:27
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: test_agent.py
import unittest

from agentuniverse.agent.agent import Agent
from agentuniverse.agent.agent_manager import AgentManager
from agentuniverse.agent.output_object import OutputObject
from agentuniverse.base.agentuniverse import AgentUniverse

AgentUniverse().start(config_path='./config.toml')


class TestAgent(unittest.TestCase):

    def test_rag_agent(self):
        instance: Agent = AgentManager().get_instance_obj('rag_agent')
        output_object: OutputObject = instance.run(input='分析下先锋领航退出中国的原因')
        print(output_object.get_data('output'))


if __name__ == '__main__':
    pass




# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/4/8 16:17
# @Author  : jerry.zzw 
# @Email   : jerry.zzw@antgroup.com
# @FileName: setup.py

import setuptools

with open("README_PYPI.md", "r") as fh:
    long_description = fh.read()


def read_requirements():
    with open('requirements.txt', 'r') as req:
        content = req.read()
        requirements = content.split('\n')

    return requirements


setuptools.setup(
    name="agentUniverse",
    version="0.0.2.beta1",
    author="AntGroup",
    author_email="jerry.zzw@antgroup.com",
    description="agentUniverse is a framework for developing applications powered "
                "by multi-agent base on large language model.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/ant-fin-agent-framework/ant-fin-agent-framework",
    packages=setuptools.find_packages(),
    package_data={
            '': ['*.yaml'],
        },
    install_requires=read_requirements(),
    classifiers=[
        'Programming Language :: Python :: 3.10',
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
)


# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/18 09:19
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: test_service.py
import pytest
from unittest.mock import patch

from agentuniverse.agent_serve.service import Service
from agentuniverse.agent_serve.service_configer import ServiceConfiger
from tests.test_agentuniverse.mock.agent_serve.mock_application_config_manager import MockApplicationConfigManager
from tests.test_agentuniverse.mock.agent_serve.mock_agent import MockAgent


@patch('agentuniverse.agent_serve.service.Agent', new=MockAgent)
@patch('agentuniverse.agent_serve.service.ApplicationConfigManager', new=MockApplicationConfigManager)
def test_service():
    agent = MockAgent({"test_result": "ok"})
    service_configer = ServiceConfiger()
    service_configer._ServiceConfiger__name = "test_service"
    service_configer._ServiceConfiger__description = "test_service"
    service_configer._ServiceConfiger__agent = agent
    service = Service(name="test_service")
    service.initialize_by_component_configer(service_configer)
    run_result = service.run()
    assert service.run() == '{"test_result": "ok"}'


if __name__ == "__main__":
    pytest.main([__file__, "-s"])



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/12 17:47
# @Author  : jerry.zzw 
# @Email   : jerry.zzw@antgroup.com
# @FileName: component_enum.py
from enum import Enum


class ComponentEnum(Enum):
    """The enumeration of the supported components."""
    AGENT = "AGENT"
    KNOWLEDGE = "KNOWLEDGE"
    LLM = "LLM"
    PLANNER = "PLANNER"
    TOOL = "TOOL"
    DEFAULT = "DEFAULT"
    SERVICE = "SERVICE"
    MEMORY = "MEMORY"

    @staticmethod
    def to_value_list():
        """Return the value list of the enumeration."""
        return [item.value for item in ComponentEnum]



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/28 11:33
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: message.py
from typing import Optional

from pydantic import BaseModel


class Message(BaseModel):
    """The basic class for memory message.

    Attributes:
        type (Optional[str]): The type of the message.
        content (Optional[str]): The content of the message.
    """

    type: Optional[str] = None
    content: Optional[str] = None



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/21 14:55
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: test_llm.py
import asyncio
import unittest

from langchain.chains import ConversationChain

from agentuniverse.llm.openai_llm import OpenAILLM


class LLMTest(unittest.TestCase):
    """
    Test cases for LLM class
    """

    def setUp(self) -> None:
        self.llm = OpenAILLM(model_name='gpt-4')

    def test_call(self) -> None:
        messages = [
            {
                "role": "user",
                "content": "hi, please introduce yourself",
            }
        ]
        output = self.llm.call(messages=messages)
        print(output.__str__())

    def test_acall(self) -> None:
        messages = [
            {
                "role": "user",
                "content": "hi, please introduce yourself",
            }
        ]
        output = asyncio.run(self.llm.acall(messages=messages))
        print(output.__str__())

    def test_as_langchain(self):
        langchain_llm = self.llm.as_langchain()
        llm_chain = ConversationChain(llm=langchain_llm)
        res = llm_chain.predict(input='hello')
        print(res)


if __name__ == '__main__':
    unittest.main()
